@@ -741,6 +741,11 @@ imageheader_row_map_fn( IMAGE *im,
 {
 	GtkWidget *hbox;
 
+	/* This is just there for back compt, hide it.
+	 */
+	if( strcmp( field, "Bbits" ) == 0 )
+		return( NULL );
+
 	hbox = imageheader_row_new( imageheader->group, field, value );
 	gtk_box_pack_start( GTK_BOX( imageheader->fields ), 
 		hbox, FALSE, FALSE, 0 );
@@ -997,155 +1002,6 @@ box_url( GtkWidget *par, const char *url )
 #endif /*lots*/
 }
 
-/* Splash screen.
- */
-
-static GtkWindowClass *splash_parent_class = NULL;
-
-static void
-splash_destroy( GtkObject *object )
-{
-	Splash *splash;
-
-	g_return_if_fail( object != NULL );
-	g_return_if_fail( IS_SPLASH( object ) );
-
-	splash = SPLASH( object );
-
-#ifdef DEBUG
-	printf( "splash_destroy\n" );
-#endif /*DEBUG*/
-
-	/* My instance destroy stuff.
-	 */
-	FREESID( splash->update_sid, progress_get() );
-
-	GTK_OBJECT_CLASS( splash_parent_class )->destroy( object );
-}
-
-static void
-splash_class_init( SplashClass *class )
-{
-	GtkObjectClass *object_class = (GtkObjectClass *) class;
-
-	splash_parent_class = g_type_class_peek_parent( class );
-
-	object_class->destroy = splash_destroy;
-}
-
-static void
-splash_progress_update( Progress *progress, gboolean *cancel, Splash *splash )
-{
-	gtk_progress_bar_set_text( GTK_PROGRESS_BAR( splash->progress ), 
-		vips_buf_all( &progress->feedback ) );
-	gtk_progress_bar_set_fraction( GTK_PROGRESS_BAR( splash->progress ), 
-		IM_CLIP( 0.0, (double) progress->percent / 100.0, 1.0 ) );
-} 
-
-static void
-splash_toggle_change_cb( GtkWidget *widget )
-{
-	if( GTK_TOGGLE_BUTTON( widget )->active ) 
-		unlinkf( "%s" G_DIR_SEPARATOR_S "%s", 
-			get_savedir(), NO_SPLASH );
-	else 
-		touchf( "%s" G_DIR_SEPARATOR_S "%s", 
-			get_savedir(), NO_SPLASH );
-}
-
-static void
-splash_init( Splash *splash )
-{
-	GtkWidget *frame;
-	GtkWidget *vbox;
-	GtkWidget *title;
-	GtkWidget *image;
-	GtkWidget *toggle;
-	char buf1[1000];
-	char buf2[1000];
-
-	gtk_window_set_type_hint( GTK_WINDOW( splash ), 
-		GDK_WINDOW_TYPE_HINT_SPLASHSCREEN );
-	gtk_window_set_position( GTK_WINDOW( splash ), 
-		GTK_WIN_POS_CENTER_ALWAYS );
-	gtk_window_set_resizable( GTK_WINDOW( splash ), FALSE );
-	gtk_window_set_decorated( GTK_WINDOW( splash ), FALSE );
-
-	frame = gtk_frame_new( NULL );
-	gtk_frame_set_shadow_type( GTK_FRAME( frame ), GTK_SHADOW_OUT );
-	gtk_container_add( GTK_CONTAINER( splash ), frame );
-
-	vbox = gtk_vbox_new( FALSE, 6 );
-        gtk_container_set_border_width( GTK_CONTAINER( vbox ), 12 );
-        gtk_container_add( GTK_CONTAINER( frame ), vbox );
-
-	title = gtk_label_new( NULL );
-	(void) im_snprintf( buf1, 1000, _( "Version %s" ), VERSION );
-	(void) im_snprintf( buf2, 1000, "<b><big>%s</big></b>\n%s", 
-		PACKAGE, buf1 );
-	gtk_label_set_markup( GTK_LABEL( title ), buf2 );
-	gtk_label_set_justify( GTK_LABEL( title ), GTK_JUSTIFY_CENTER );
-        gtk_box_pack_start( GTK_BOX( vbox ), title, FALSE, FALSE, 0 );
-
-	image = image_new_from_file( 
-		"$VIPSHOME/share/$PACKAGE/data/vips-128.png" );
-        gtk_box_pack_start( GTK_BOX( vbox ), image, FALSE, FALSE, 0 );
-
-	splash->progress = gtk_progress_bar_new();
-	gtk_progress_bar_set_text( GTK_PROGRESS_BAR( splash->progress ), 
-		_( "Starting ..." ) );
-        gtk_box_pack_start( GTK_BOX( vbox ), 
-		splash->progress, FALSE, FALSE, 0 );
-	splash->update_sid = g_signal_connect( progress_get(), "update", 
-		G_CALLBACK( splash_progress_update ), splash );
-        gtk_widget_show( splash->progress );
-
-        toggle = build_gtoggle( GTK_WIDGET( vbox ), 
-		_( "Display this splash screen during startup" ) );
-	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( toggle ), TRUE );
-        gtk_signal_connect( GTK_OBJECT( toggle ), "clicked",
-		GTK_SIGNAL_FUNC( splash_toggle_change_cb ), NULL );
-
-	/* Make sure the "don't show" widget is unfocussed ... we don't want
-	 * an accidental ENTER killing it.
-	 */
-	GTK_WIDGET_SET_FLAGS( image, GTK_CAN_FOCUS );
-	gtk_widget_grab_focus( image );
-
-	gtk_widget_show_all( frame );
-}
-
-GtkType
-splash_get_type( void )
-{
-	static GtkType splash_type = 0;
-
-	if( !splash_type ) {
-		static const GtkTypeInfo info = {
-			"Splash",
-			sizeof( Splash ),
-			sizeof( SplashClass ),
-			(GtkClassInitFunc) splash_class_init,
-			(GtkObjectInitFunc) splash_init,
-			/* reserved_1 */ NULL,
-			/* reserved_2 */ NULL,
-			(GtkClassInitFunc) NULL,
-		};
-
-		splash_type = gtk_type_unique( GTK_TYPE_WINDOW, &info );
-	}
-
-	return( splash_type );
-}
-
-Splash *
-splash_new( void )
-{
-	Splash *splash = gtk_type_new( TYPE_SPLASH );
-
-	return( splash );
-}
-
 /* Fontchooser dialog.
  */
 
