@@ -184,9 +184,9 @@ apply_complex_call( Reduce *rc,
 
 	if( PEISIMAGE( &rhs ) ) {
 		if( strcmp( name, "re" ) == 0 ) 
-			vips_spine( rc, "im_c2real", arg, out );
+			call_spine( rc, "im_c2real", arg, out );
 		else if( strcmp( name, "im" ) == 0 ) 
-			vips_spine( rc, "im_c2imag", arg, out );
+			call_spine( rc, "im_c2imag", arg, out );
 	}
 	else if( PEISCOMPLEX( &rhs ) ) {
 		if( strcmp( name, "re" ) == 0 ) {
@@ -284,8 +284,11 @@ apply_image_call( Reduce *rc,
 	const char *name, HeapNode **arg, PElement *out )
 {
 	Heap *heap = rc->heap;
+
 	PElement rhs;
 	char buf[FILENAME_MAX];
+	char filename[FILENAME_MAX];
+	char mode[FILENAME_MAX];
 	char *fn;
 	Imageinfo *ii;
 
@@ -294,12 +297,21 @@ apply_image_call( Reduce *rc,
 	PEPOINTRIGHT( arg[0], &rhs );
 	(void) reduce_get_string( rc, &rhs, buf, FILENAME_MAX );
 
+	/* The buf might be something like n3862.pyr.tif:1, ie. contain some
+	 * load options. Split and search just for the filename component.
+	 */
+	im_filename_split( buf, filename, mode );
+
 	/* Try to load image from given string.
 	 */
-	if( !(fn = path_find_file( PATH_SEARCH, buf )) )
+	if( !(fn = path_find_file( PATH_SEARCH, filename )) )
 		reduce_throw( rc );
+
+	/* Reattach the mode and load.
+	 */
+	im_snprintf( buf, FILENAME_MAX, "%s:%s", fn, mode );
 	if( !(ii = imageinfo_new_input( 
-		main_imageinfogroup, NULL, heap, fn )) ) {
+		main_imageinfogroup, NULL, heap, buf )) ) {
 		IM_FREE( fn );
 		reduce_throw( rc );
 	}
@@ -397,7 +409,7 @@ apply_math_call( Reduce *rc,
 	if( PEISIMAGE( &rhs ) ) {
 		/* Easy ... pass to VIPS.
 		 */
-		vips_spine( rc, math_fn[i].ifn, arg, out );
+		call_spine( rc, math_fn[i].ifn, arg, out );
 	}
 	else if( PEISREAL( &rhs ) ) {
 		double a = PEGETREAL( &rhs );
@@ -509,7 +521,9 @@ apply_search_call( Reduce *rc, const char *name, HeapNode **arg, PElement *out )
 	(void) reduce_get_string( rc, &rhs, buf, MAX_STRSIZE );
 
 	if( !(fn = path_find_file( PATH_SEARCH, buf )) )
-		reduce_throw( rc );
+		/* If not found, return [].
+		 */
+		fn = im_strdup( NULL, "" );
 
 	if( !heap_managedstring_new( rc->heap, fn, out ) ) {
 		IM_FREE( fn );
@@ -733,6 +747,12 @@ apply_name2gtype_call( Reduce *rc, const char *name,
 
 	PEPOINTRIGHT( arg[0], &rhs );
 	(void) reduce_get_string( rc, &rhs, txt, FILENAME_MAX );
+
+	/* Hack ... map the old vips7 names to the new ones.
+	 */
+	if( strcmp( txt, "im_ref_string" ) == 0 )
+		strcpy( txt, "vips_ref_string" );
+
 	gtype = g_type_from_name( txt );
 
 	if( !heap_real_new( rc->heap, gtype, out ) )
