@@ -119,6 +119,21 @@ filemodel_set_modified( Filemodel *filemodel, gboolean modified )
 		filemodel_class->set_modified( filemodel, modified );
 }
 
+void 
+filemodel_set_window_hint( Filemodel *filemodel, iWindow *iwnd )
+{
+	filemodel->window_hint = iwnd;
+}
+
+iWindow * 
+filemodel_get_window_hint( Filemodel *filemodel )
+{
+	if( filemodel->window_hint )
+		return( filemodel->window_hint );
+	else
+		return( IWINDOW( main_window_top ) );
+}
+
 gboolean
 filemodel_save_all( Filemodel *filemodel, const char *filename )
 {
@@ -305,15 +320,15 @@ filemodel_save_all_text( Filemodel *filemodel, const char *filename )
 {
 	iOpenFile *of;
 
-	if( !(of = file_open_write( "%s", filename )) ) 
+	if( !(of = ifile_open_write( "%s", filename )) ) 
 		return( FALSE );
 
 	column_set_offset( filemodel->x_off, filemodel->y_off );
 	if( model_save_text( MODEL( filemodel ), of ) ) {
-		file_close( of );
+		ifile_close( of );
 		return( FALSE );
 	}
-	file_close( of );
+	ifile_close( of );
 
 	return( TRUE );
 }
@@ -394,6 +409,8 @@ filemodel_init( Filemodel *filemodel )
 	filemodel->major = MAJOR_VERSION;
 	filemodel->minor = MINOR_VERSION;
 	filemodel->micro = MICRO_VERSION;
+
+	filemodel->window_hint = NULL;
 }
 
 GtkType
@@ -526,14 +543,14 @@ filemodel_load_all_text( Filemodel *filemodel,
 {
 	iOpenFile *of;
 
-	if( !(of = file_open_read( "%s", filename )) ) 
+	if( !(of = ifile_open_read( "%s", filename )) ) 
 		return( FALSE );
 
 	if( model_load_text( MODEL( filemodel ), parent, of ) ) {
-		file_close( of );
+		ifile_close( of );
 		return( FALSE );
 	}
-	file_close( of );
+	ifile_close( of );
 
 	return( TRUE );
 }
@@ -885,7 +902,8 @@ filemodel_inter_close_registered_cb( iWindow *iwnd, void *client,
 			filemodel_inter_close_registered_cb, 
 			iwnd, client, nfn, sys );
 
-		filemodel_inter_savenclose_cb( iwnd, filemodel, 
+		filemodel_inter_savenclose_cb( 
+			filemodel_get_window_hint( filemodel ), filemodel, 
 			iwindow_susp_comp, susp );
 	}
 	else
