@@ -108,10 +108,11 @@ iimage_info( iObject *iobject, VipsBuf *buf )
 	if( ii && (im = imageinfo_get( FALSE, ii )) ) {
 		char *filename;
 
-		if( im_header_get_type( im, ORIGINAL_FILENAME ) != 0 ) {
+		if( im_header_get_typeof( im, ORIGINAL_FILENAME ) != 0 ) {
 			if( !im_header_string( im, 
 				ORIGINAL_FILENAME, &filename ) ) {
-				vips_buf_appends( buf, _( "Original filename" ) );
+				vips_buf_appends( buf, 
+					_( "Original filename" ) );
 				vips_buf_appendf( buf, ": %s\n", filename );
 			}
 		}
@@ -260,7 +261,7 @@ iimage_class_get( Classmodel *classmodel, PElement *root )
 		char *filename;
 
 		if( (im = imageinfo_get( FALSE, ii )) &&
-			im_header_get_type( im, ORIGINAL_FILENAME ) != 0 ) {
+			im_header_get_typeof( im, ORIGINAL_FILENAME ) != 0 ) {
 			if( im_header_string( im, 
 				ORIGINAL_FILENAME, &filename ) )
 				return( FALSE );
@@ -349,15 +350,15 @@ iimage_replace( iImage *iimage, const char *filename )
 	Row *row = HEAPMODEL( iimage )->row;
 	iText *itext = ITEXT( HEAPMODEL( iimage )->rhs->itext );
 	char txt[MAX_STRSIZE];
-	VipsBuf buf;
+	VipsBuf buf = VIPS_BUF_STATIC( txt );
 
-	vips_buf_init_static( &buf, txt, MAX_STRSIZE );
 	vips_buf_appends( &buf, "Image_file \"" );
 	vips_buf_appendsc( &buf, TRUE, filename );
 	vips_buf_appends( &buf, "\"" );
 
 	if( itext_set_formula( itext, vips_buf_all( &buf ) ) ) {
 		itext_set_edited( itext, TRUE );
+		filemodel_set_modified( FILEMODEL( row->ws ), TRUE );
 		(void) expr_dirty( row->expr, link_serial_new() );
 
 		mainw_recent_add( &mainw_recent_image, filename );
