@@ -200,7 +200,7 @@ extern int statfs();
 				/* XML namespace ... note, not nip2! */
 #define MAX_LINELENGTH (120)	/* Max chars we display of value */
 #define MAX_RECENT (10)		/* Number of recent items in file menu */
-#define NIP_COPYRIGHT "%s: &#0169;2010 Imperial College, London"
+#define NIP_COPYRIGHT "%s: &#0169;2011 Imperial College, London"
 
 /* Our stock_ids.
  */
@@ -480,8 +480,8 @@ typedef void *(*tool_map_fn)( Tool *, void *, void * );
 #include "itextview.h"
 #include "toggle.h"
 #include "toggleview.h"
-#include "vips_call.h"
-#include "vips_cache.h"
+#include "call.h"
+#include "cache.h"
 #include "parser.h"
 
 #ifdef WITH_DMALLOC
